/**********************************************************
* Evolventenberechnungs-Programm fuer TI84 Plus CE-T
* mit CEDEV --> Z80 Assembly und C
* z.B. windows_CEdev.exe auf Github
* 2.4.2020 Rieg
* 1.5.2020 Rieg: atan fuer 
**********************************************************/
#include <stdbool.h>
#include <stdlib.h>
#include <stddef.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <tice.h>
#include <math.h>

#include <graphx.h>

/***********************************************************************
* globale Variable
***********************************************************************/
/*----------------------------------------------------------------------
* zu den Datentypen:
* float entspricht double, 32 Bit
* int entspricht long, 24 Bit
*---------------------------------------------------------------------*/
float dy[13],syn[13],hy[10],by[10];
float galfan,ralfan,ralfat,gbeta,rbeta,rbetab,ralfavt,rbetav,gpsiv;
float z,m,x,d,db,dv,psi,psiv,da,te1,te2,dr,dinc;
float xmin,xmax,ymin,ymax,fsn,fdy,x1,x2,y1,y2;
float pi= 3.141593;

int i,is,ikey;

char cstring[32],cresult[48];  // lieber etwas groesser

/***********************************************************************
* Spezial-Functions print, float2str und str2float
* Den direkten Zugriff via scanf und printf gibt's nicht.
***********************************************************************/
/*----------------------------------------------------------------------
*  print
*---------------------------------------------------------------------*/
void print(const char* text, uint8_t xpos, uint8_t ypos) 
  {
  os_SetCursorPos(ypos, xpos);
  os_PutStrFull(text);
  }

/*----------------------------------------------------------------------
*  float2str wandelt Floats in Strings um, muss über TI84-Reals laufen,
*  sprintf, ftoa etc. funktionieren nicht bzw. nicht zuverlaessig.
*  Function ist sinngemaess aus dem Internet:
*  https://github.com/CE-Programming/toolchain
*---------------------------------------------------------------------*/
void float2str(float wert,char *cstr)
  {
  real_t  ti84real;

  ti84real= os_FloatToReal(wert);
  os_RealToStr(cstr,&ti84real,0,1,-1); 
  // nach https://ce-programming.github.io/toolchain/tice_8h.html
  //  0= use default max length (14) 
  //  1= Normal mode 
  // -1= Float mode 
  }

/*----------------------------------------------------------------------
*  str2float wandelt Strings in Floats um, muss über TI84-Reals laufen,
*  sprintf, atof etc. funktionieren nicht bzw. nicht zuverlaessig.
*  Dagegen scheint atoi sauber zu laufen.
*  Function ist ein Entwurf von F.Rieg
*---------------------------------------------------------------------*/
float str2float(char *cstr)
  {
  float   wert;
  char    **end;     // nach tice: "If non-null, pointer to end 
                     // of parsed number is stored here" 
  char    *cptr[3];  // damit es keine Warning gibt
  real_t  ti84real;

  end= &cptr;        // damit es keine Warning gibt

  ti84real= os_StrToReal(cstr,end);
  wert= os_RealToFloat(&ti84real);
  return(wert);
  }
  
/***********************************************************************
* eigene Functions
***********************************************************************/
float inv(float);
float g2r(float);
float r2g(float);
float rund(float);

/***********************************************************************
* main
***********************************************************************/
void main(void) 
{
/*----------------------------------------------------------------------
*  Startwerte
*---------------------------------------------------------------------*/
m     = 2.5;
x     = 0.12;
z     = 28;
galfan= 20.;
gbeta = 0.;

is= 6;  // 6 Messwerte hy,by

hy[1]=1.5;
by[1]=3.29;
hy[2]=2.;
by[2]=3.67;
hy[3]=2.5;
by[3]=4.08;
hy[4]=3.;
by[4]=4.32;
hy[5]=3.5;
by[5]=4.57;
hy[6]=4.;
by[6]=4.89;

/*----------------------------------------------------------------------
*  Lbl start
*---------------------------------------------------------------------*/
start:;

os_ClrHome();

print("Evolventenberechnung,9 Info",0,0);
print("1 Zaehnezahl z",0,1);
print("2 Modul m",0,2);
print("3 Profilver. x",0,3);
print("4 Winkel alfan + beta",0,4);
print("5 Messwerte",0,5);
print("6 momentan def. Werte",0,6);
print("7 Compute",0,7);
print("8 Exit",0,8);

os_SetCursorPos(9, 0);
os_GetStringInput("Ihre Wahl? ",cstring,20);
ikey=atoi(cstring);

/*----------------------------------------------------------------------
* i Info
*---------------------------------------------------------------------*/
if(ikey == 9)
  {
  os_ClrHome();

  print("YEVOL MK2,Lehrstuhl Konstruktionslehre+CAD,Universitaet Bayreuth,\
Prof.Frank Rieg,1.5.2020,Evolventenberechnung nach DIN 3960 bzw. Linke",0,0);

  os_SetCursorPos(9,0);
  os_GetStringInput("weiter mit Enter-Taste..",cstring,20);
  ikey=atoi(cstring);

  goto start;
  }

/*----------------------------------------------------------------------
*  F8 Exit
*---------------------------------------------------------------------*/
if(ikey == 8)goto quit;

/*----------------------------------------------------------------------
*  F1 Zaehnezahl
*---------------------------------------------------------------------*/
if(ikey == 1)
  {
  os_ClrHome();

  print("Zaehnezahl:",0,0);

  os_SetCursorPos(1,0);
  os_GetStringInput("z? ",cstring,20);
  z= str2float(cstring);

  goto start;
  }

/*----------------------------------------------------------------------
*  F2 Modul
*---------------------------------------------------------------------*/
if(ikey == 2)
  {
  os_ClrHome();

  print("Modul:",0,0);
  
  os_SetCursorPos(1,0);
  os_GetStringInput("m(mm)? ",cstring,20);
  m= str2float(cstring);

  goto start;
  }

/*----------------------------------------------------------------------
*  F3 Profilverschiebung
*---------------------------------------------------------------------*/
if(ikey == 3)
  {
  os_ClrHome();

  print("Profilverschiebung:",0,0);

  os_SetCursorPos(1,0);
  os_GetStringInput("x? ",cstring,20);
  x= str2float(cstring);

  goto start;
}

/*----------------------------------------------------------------------
*  F4  Winkel: Alfan und beta
*---------------------------------------------------------------------*/
if(ikey == 4)
  {
  os_ClrHome();

  print("Eingriffswinkel:",0,0);

  os_SetCursorPos(1,0);
  os_GetStringInput("alfan(Grad)? ",cstring,20);
  galfan= str2float(cstring);

  print("Schraegungswinkel:",0,2);

  os_SetCursorPos(3,0);
  os_GetStringInput("beta(Grad)? ",cstring,20);
  gbeta= str2float(cstring);

  goto start;
  }

/*----------------------------------------------------------------------
*  F5  Messwerte
*---------------------------------------------------------------------*/
if(ikey == 5)
  {
  os_ClrHome();

  print("Messwerte hy und syn:",0,0);

    do
    {
    os_SetCursorPos(2,0);
    os_GetStringInput("Anzahl Messw.(max.8)? ",cstring,20);
    is= atoi(cstring);
    }
  while(is < 0 || is > 8);

  for(i=1;i<=is;i++)
    {
    float2str((float)i,cstring);
    strcpy(cresult,"i= ");
    strcat(cresult,cstring);
    print(cresult,0,4);

    os_SetCursorPos(5,0);
    os_GetStringInput("hy(mm)? ",cstring,20);
    hy[i]= str2float(cstring);

    os_SetCursorPos(6,0);
    os_GetStringInput("syn(mm)? ",cstring,20);
    by[i]= str2float(cstring);
    }

  goto start;
  }

/*----------------------------------------------------------------------
*  F6 momentan definierte Werte
*---------------------------------------------------------------------*/
if(ikey == 6)
  {
  os_ClrHome();

  print("momentan def. Werte 1:",0,0);

  float2str(z,cstring);
  strcpy(cresult,"Zaehnezahl= ");
  strcat(cresult,cstring);
  print(cresult,0,1);
  
  float2str(m,cstring);
  strcpy(cresult,"Modul= ");
  strcat(cresult,cstring);
  print(cresult,0,2);

  float2str(x,cstring);
  strcpy(cresult,"Profilvers.= ");
  strcat(cresult,cstring);
  print(cresult,0,3);

  float2str(galfan,cstring);
  strcpy(cresult,"Eingriffsw.= ");
  strcat(cresult,cstring);
  print(cresult,0,4);

  float2str(gbeta,cstring);
  strcpy(cresult,"Schraegungsw.= ");
  strcat(cresult,cstring);
  print(cresult,0,5);

  float2str((float)is,cstring);
  strcpy(cresult,"Anzahl Messwerte= ");
  strcat(cresult,cstring);
  print(cresult,0,6);

  os_SetCursorPos(9,0);
  os_GetStringInput("weiter mit Enter-Taste..",cstring,20);

  os_ClrHome();

  if(is == 0) goto start; // keine Messwerte: skip Ausgabe 2

  print("momentan def. Werte 2:",0,0);

   for(i=1;i<=is;i++)
    {
    float2str((float)i,cresult);
    strcat(cresult,":hy=");

    float2str(hy[i],cstring);
    strcat(cresult,cstring);
    strcat(cresult,",syn=");

    float2str(by[i],cstring);
    strcat(cresult,cstring);
    print(cresult,0,i);
    }
  os_SetCursorPos(9,0);
  os_GetStringInput("weiter mit Enter-Taste..",cstring,20);

  os_ClrHome();


  goto start;
  }

/*----------------------------------------------------------------------
*  F7 Compute
*---------------------------------------------------------------------*/
if(ikey == 7)
  {
  os_ClrHome();

  rbeta= g2r(gbeta);
  ralfan= g2r(galfan);

  rbetab= asin(sin(rbeta)*cos(ralfan));
  ralfat= atan(tan(ralfan)/cos(rbeta));

  d = z*m/cos(rbeta);
  db= d*cos(ralfat);
  dv= d+2.*x*m;
  da= dv+2.*m;

/*======================================================================
*  Schleife, 11 Schritte von db bis da
*=====================================================================*/
  dinc= (da-db)/10.;

  for(i=1; i<=11;i++)
    {
    dy[i]= db+(i-1)*dinc;
    dr= dy[i];

    ralfavt= acos(db/dr);
    rbetav= atan(dr/d*tan(rbeta));  // hier atan eingebaut, 1.5.20

    psi= (pi+4.*x*tan(ralfan))/(2.*z);
    psiv= psi+(inv(ralfat)-inv(ralfavt));

    te1= pow((psiv*cos(rbetav)*sin(rbetav)),2.);
    te2= pow(sin(psiv*cos(rbetav)*cos(rbetav)),2.);
    syn[i]= dr*sqrt(te1+te2);
    }

/*======================================================================
*  Ergebnisse andrucken
*=====================================================================*/
  print("Ergebnisse 1:",0,0);

  float2str(db,cstring);
  strcpy(cresult,"db  = ");
  strcat(cresult,cstring);
  print(cresult,0,1);

  float2str(dv,cstring);
  strcpy(cresult,"dv  = ");
  strcat(cresult,cstring);
  print(cresult,0,2);

  float2str(da,cstring);
  strcpy(cresult,"da  = ");
  strcat(cresult,cstring);
  print(cresult,0,3);

  os_SetCursorPos(9,0);
  os_GetStringInput("weiter mit Enter-Taste",cstring,20);

  os_ClrHome();

  print("Ergebnisse 2:",0,0);

  float2str(dy[1],cstring);
  strcpy(cresult,"dy1  = ");
  strcat(cresult,cstring);
  print(cresult,0,1);

  float2str(syn[1],cstring);
  strcpy(cresult,"syn1 = ");
  strcat(cresult,cstring);
  print(cresult,0,2);

  float2str(dy[2],cstring);
  strcpy(cresult,"dy2  = ");
  strcat(cresult,cstring);
  print(cresult,0,3);

  float2str(syn[2],cstring);
  strcpy(cresult,"syn2 = ");
  strcat(cresult,cstring);
  print(cresult,0,4);

  float2str(dy[3],cstring);
  strcpy(cresult,"dy3  = ");
  strcat(cresult,cstring);
  print(cresult,0,5);

  float2str(syn[3],cstring);
  strcpy(cresult,"syn3 = ");
  strcat(cresult,cstring);
  print(cresult,0,6);

  float2str(dy[4],cstring);
  strcpy(cresult,"dy4  = ");
  strcat(cresult,cstring);
  print(cresult,0,7);

  float2str(syn[4],cstring);
  strcpy(cresult,"syn4 = ");
  strcat(cresult,cstring);
  print(cresult,0,8);

  os_SetCursorPos(9,0);
  os_GetStringInput("weiter mit Enter-Taste",cstring,20);

  os_ClrHome();

  print("Ergebnisse 3:",0,0);

  float2str(dy[5],cstring);
  strcpy(cresult,"dy5  = ");
  strcat(cresult,cstring);
  print(cresult,0,1);

  float2str(syn[5],cstring);
  strcpy(cresult,"syn5 = ");
  strcat(cresult,cstring);
  print(cresult,0,2);

  float2str(dy[6],cstring);
  strcpy(cresult,"dy6  = ");
  strcat(cresult,cstring);
  print(cresult,0,3);

  float2str(syn[6],cstring);
  strcpy(cresult,"syn6 = ");
  strcat(cresult,cstring);
  print(cresult,0,4);

  float2str(dy[7],cstring);
  strcpy(cresult,"dy7  = ");
  strcat(cresult,cstring);
  print(cresult,0,5);

  float2str(syn[7],cstring);
  strcpy(cresult,"syn7 = ");
  strcat(cresult,cstring);
  print(cresult,0,6);

  float2str(dy[8],cstring);
  strcpy(cresult,"dy8  = ");
  strcat(cresult,cstring);
  print(cresult,0,7);

  float2str(syn[8],cstring);
  strcpy(cresult,"syn8 = ");
  strcat(cresult,cstring);
  print(cresult,0,8);

  os_SetCursorPos(9,0);
  os_GetStringInput("weiter mit Enter-Taste",cstring,20);

  os_ClrHome();

  print("Ergebnisse 4:",0,0);

  float2str(dy[9],cstring);
  strcpy(cresult,"dy9  = ");
  strcat(cresult,cstring);
  print(cresult,0,1);

  float2str(syn[9],cstring);
  strcpy(cresult,"syn9 = ");
  strcat(cresult,cstring);
  print(cresult,0,2);

  float2str(dy[10],cstring);
  strcpy(cresult,"dy10  = ");
  strcat(cresult,cstring);
  print(cresult,0,3);

  float2str(syn[10],cstring);
  strcpy(cresult,"syn10 = ");
  strcat(cresult,cstring);
  print(cresult,0,4);

  float2str(dy[11],cstring);
  strcpy(cresult,"dy11 = ");
  strcat(cresult,cstring);
  print(cresult,0,5);

  float2str(syn[11],cstring);
  strcpy(cresult,"syn11= ");
  strcat(cresult,cstring);
  print(cresult,0,6);

  os_SetCursorPos(9,0);
  os_GetStringInput("weiter mit Enter-Taste",cstring,20);

  os_ClrHome();

/*======================================================================
*  Graphik
*=====================================================================*/
  gfx_Begin();
  gfx_SetDrawBuffer();
  gfx_FillScreen(255);  // weiss

  /* Bildschirmgroesse, y waechst von oben nach unten */
  xmin= 0;
  xmax= 319;
  ymin= 239;
  ymax= 0;

  for(i=1;i<=11;i++)
    syn[i]/=2.;      // halbe Breite
  
  /* 80 ist 1/4 von 319 */
  fsn= 80/syn[1];
  fdy= fsn;

  /* die berechnete Evolvente malen */
  gfx_SetColor(0);  // schwarz
  gfx_SetTextFGColor(0);
  gfx_SetTextXY(10,10);
  gfx_PrintString("Schwarz: Berechnete Evolvente");
  gfx_SetTextXY(10,20);
  gfx_PrintString("Evolvente laeuft von db bis da.");
  
  for(i=1;i<11;i++)
    {
    /* Malen der linken Kurve */
    x1= 160 - syn[i]*fsn;  // syn ist hier nur noch halbe Breite
    y1= 239 - (dy[i]-dy[1])*0.5*fdy; // Durchmesserdifferenz/2
    x2= 160 - syn[i+1]*fsn;
    y2= 239 - (dy[i+1]-dy[1])*0.5*fdy;
    gfx_Line(x1,y1,x2,y2);
    }

  for(i=1;i<11;i++)
    {
    /* Malen der rechten Kurve */
    x1= 160 + syn[i]*fsn;  // syn ist hier nur noch halbe Breite
    y1= 239 - (dy[i]-dy[1])*0.5*fdy; // Durchmesserdifferenz/2
    x2= 160 + syn[i+1]*fsn;
    y2= 239 - (dy[i+1]-dy[1])*0.5*fdy;
    gfx_Line(x1,y1,x2,y2);
    }

  /* Malen der oberen Verbindungslinie */
  x1= 160 - syn[11]*fsn;
  y1= 239 - (dy[11]-dy[1])*0.5*fdy;
  x2= 160 + syn[11]*fsn;
  y2= 239 - (dy[11]-dy[1])*0.5*fdy;

  gfx_Line(x1,y1,x2,y2);

  /* Malen der gemessenen Werte */
  gfx_SetColor(224);  // rot
  gfx_SetTextFGColor(224);
  gfx_SetTextXY(10,30);
  gfx_PrintString("Rot: Gemessene Werte");

  for(i=1;i<=is;i++)
    {
    /* Malen der Messpunkte */
    x1= 160 + by[i]*0.5*fsn;   // halbe Breite
    y1= 239 - ((da-db)*0.5-hy[i])*fdy;
    gfx_FillCircle(x1,y1,3);
    }

  gfx_BlitBuffer();

  while (!os_GetCSC());
  gfx_End();

  goto start;
}

/*----------------------------------------------------------------------
*  Lbl quit
----------------------------------------------------------------------*/
goto start;  /* falsche Taste */
quit:;
}

/***********************************************************************
* Involut-Funktion
***********************************************************************/
float inv(float gamma)
{
return (tan(gamma)-gamma);
}

/***********************************************************************
* Grad-to-Bogenmass-Funktion
***********************************************************************/
float g2r(float gamma)
{
return(gamma*pi/180);
}

/***********************************************************************
* Bogenmass-to-Grad-Funktion
***********************************************************************/
float r2g(float gamma)
{
return(gamma*180/pi);
}

/***********************************************************************
* Runden-Funktion
***********************************************************************/
float rund(float gamma)
{
return(floor(gamma+0.5));
}

